"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _Stack_js_1 = require("./_Stack.js");
var _arrayEach_js_1 = require("./_arrayEach.js");
var _assignValue_js_1 = require("./_assignValue.js");
var _baseAssign_js_1 = require("./_baseAssign.js");
var _baseAssignIn_js_1 = require("./_baseAssignIn.js");
var _cloneBuffer_js_1 = require("./_cloneBuffer.js");
var _copyArray_js_1 = require("./_copyArray.js");
var _copySymbols_js_1 = require("./_copySymbols.js");
var _copySymbolsIn_js_1 = require("./_copySymbolsIn.js");
var _getAllKeys_js_1 = require("./_getAllKeys.js");
var _getAllKeysIn_js_1 = require("./_getAllKeysIn.js");
var _getTag_js_1 = require("./_getTag.js");
var _initCloneArray_js_1 = require("./_initCloneArray.js");
var _initCloneByTag_js_1 = require("./_initCloneByTag.js");
var _initCloneObject_js_1 = require("./_initCloneObject.js");
var isArray_js_1 = require("./isArray.js");
var isBuffer_js_1 = require("./isBuffer.js");
var isObject_js_1 = require("./isObject.js");
var keys_js_1 = require("./keys.js");
var CLONE_DEEP_FLAG = 1, CLONE_FLAT_FLAG = 2, CLONE_SYMBOLS_FLAG = 4;
var argsTag = '[object Arguments]', arrayTag = '[object Array]', boolTag = '[object Boolean]', dateTag = '[object Date]', errorTag = '[object Error]', funcTag = '[object Function]', genTag = '[object GeneratorFunction]', mapTag = '[object Map]', numberTag = '[object Number]', objectTag = '[object Object]', regexpTag = '[object RegExp]', setTag = '[object Set]', stringTag = '[object String]', symbolTag = '[object Symbol]', weakMapTag = '[object WeakMap]';
var arrayBufferTag = '[object ArrayBuffer]', dataViewTag = '[object DataView]', float32Tag = '[object Float32Array]', float64Tag = '[object Float64Array]', int8Tag = '[object Int8Array]', int16Tag = '[object Int16Array]', int32Tag = '[object Int32Array]', uint8Tag = '[object Uint8Array]', uint8ClampedTag = '[object Uint8ClampedArray]', uint16Tag = '[object Uint16Array]', uint32Tag = '[object Uint32Array]';
var cloneableTags = {};
cloneableTags[argsTag] = cloneableTags[arrayTag] =
    cloneableTags[arrayBufferTag] = cloneableTags[dataViewTag] =
        cloneableTags[boolTag] = cloneableTags[dateTag] =
            cloneableTags[float32Tag] = cloneableTags[float64Tag] =
                cloneableTags[int8Tag] = cloneableTags[int16Tag] =
                    cloneableTags[int32Tag] = cloneableTags[mapTag] =
                        cloneableTags[numberTag] = cloneableTags[objectTag] =
                            cloneableTags[regexpTag] = cloneableTags[setTag] =
                                cloneableTags[stringTag] = cloneableTags[symbolTag] =
                                    cloneableTags[uint8Tag] = cloneableTags[uint8ClampedTag] =
                                        cloneableTags[uint16Tag] = cloneableTags[uint32Tag] = true;
cloneableTags[errorTag] = cloneableTags[funcTag] =
    cloneableTags[weakMapTag] = false;
function baseClone(value, bitmask, customizer, key, object, stack) {
    var result, isDeep = bitmask & CLONE_DEEP_FLAG, isFlat = bitmask & CLONE_FLAT_FLAG, isFull = bitmask & CLONE_SYMBOLS_FLAG;
    if (customizer) {
        result = object ? customizer(value, key, object, stack) : customizer(value);
    }
    if (result !== undefined) {
        return result;
    }
    if (!isObject_js_1.default(value)) {
        return value;
    }
    var isArr = isArray_js_1.default(value);
    if (isArr) {
        result = _initCloneArray_js_1.default(value);
        if (!isDeep) {
            return _copyArray_js_1.default(value, result);
        }
    }
    else {
        var tag = _getTag_js_1.default(value), isFunc = tag == funcTag || tag == genTag;
        if (isBuffer_js_1.default(value)) {
            return _cloneBuffer_js_1.default(value, isDeep);
        }
        if (tag == objectTag || tag == argsTag || (isFunc && !object)) {
            result = (isFlat || isFunc) ? {} : _initCloneObject_js_1.default(value);
            if (!isDeep) {
                return isFlat
                    ? _copySymbolsIn_js_1.default(value, _baseAssignIn_js_1.default(result, value))
                    : _copySymbols_js_1.default(value, _baseAssign_js_1.default(result, value));
            }
        }
        else {
            if (!cloneableTags[tag]) {
                return object ? value : {};
            }
            result = _initCloneByTag_js_1.default(value, tag, baseClone, isDeep);
        }
    }
    stack || (stack = new _Stack_js_1.default);
    var stacked = stack.get(value);
    if (stacked) {
        return stacked;
    }
    stack.set(value, result);
    var keysFunc = isFull
        ? (isFlat ? _getAllKeysIn_js_1.default : _getAllKeys_js_1.default)
        : (isFlat ? keysIn : keys_js_1.default);
    var props = isArr ? undefined : keysFunc(value);
    _arrayEach_js_1.default(props || value, function (subValue, key) {
        if (props) {
            key = subValue;
            subValue = value[key];
        }
        _assignValue_js_1.default(result, key, baseClone(subValue, bitmask, customizer, key, value, stack));
    });
    return result;
}
exports.default = baseClone;
