"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _composeArgs_js_1 = require("./_composeArgs.js");
var _composeArgsRight_js_1 = require("./_composeArgsRight.js");
var _countHolders_js_1 = require("./_countHolders.js");
var _createCtor_js_1 = require("./_createCtor.js");
var _createRecurry_js_1 = require("./_createRecurry.js");
var _getHolder_js_1 = require("./_getHolder.js");
var _reorder_js_1 = require("./_reorder.js");
var _replaceHolders_js_1 = require("./_replaceHolders.js");
var _root_js_1 = require("./_root.js");
var WRAP_BIND_FLAG = 1, WRAP_BIND_KEY_FLAG = 2, WRAP_CURRY_FLAG = 8, WRAP_CURRY_RIGHT_FLAG = 16, WRAP_ARY_FLAG = 128, WRAP_FLIP_FLAG = 512;
function createHybrid(func, bitmask, thisArg, partials, holders, partialsRight, holdersRight, argPos, ary, arity) {
    var isAry = bitmask & WRAP_ARY_FLAG, isBind = bitmask & WRAP_BIND_FLAG, isBindKey = bitmask & WRAP_BIND_KEY_FLAG, isCurried = bitmask & (WRAP_CURRY_FLAG | WRAP_CURRY_RIGHT_FLAG), isFlip = bitmask & WRAP_FLIP_FLAG, Ctor = isBindKey ? undefined : _createCtor_js_1.default(func);
    function wrapper() {
        var length = arguments.length, args = Array(length), index = length;
        while (index--) {
            args[index] = arguments[index];
        }
        if (isCurried) {
            var placeholder = _getHolder_js_1.default(wrapper), holdersCount = _countHolders_js_1.default(args, placeholder);
        }
        if (partials) {
            args = _composeArgs_js_1.default(args, partials, holders, isCurried);
        }
        if (partialsRight) {
            args = _composeArgsRight_js_1.default(args, partialsRight, holdersRight, isCurried);
        }
        length -= holdersCount;
        if (isCurried && length < arity) {
            var newHolders = _replaceHolders_js_1.default(args, placeholder);
            return _createRecurry_js_1.default(func, bitmask, createHybrid, wrapper.placeholder, thisArg, args, newHolders, argPos, ary, arity - length);
        }
        var thisBinding = isBind ? thisArg : this, fn = isBindKey ? thisBinding[func] : func;
        length = args.length;
        if (argPos) {
            args = _reorder_js_1.default(args, argPos);
        }
        else if (isFlip && length > 1) {
            args.reverse();
        }
        if (isAry && ary < length) {
            args.length = ary;
        }
        if (this && this !== _root_js_1.default && this instanceof wrapper) {
            fn = Ctor || _createCtor_js_1.default(fn);
        }
        return fn.apply(thisBinding, args);
    }
    return wrapper;
}
exports.default = createHybrid;
