"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _baseSetData_js_1 = require("./_baseSetData.js");
var _createBind_js_1 = require("./_createBind.js");
var _createCurry_js_1 = require("./_createCurry.js");
var _createHybrid_js_1 = require("./_createHybrid.js");
var _createPartial_js_1 = require("./_createPartial.js");
var _getData_js_1 = require("./_getData.js");
var _mergeData_js_1 = require("./_mergeData.js");
var _setData_js_1 = require("./_setData.js");
var _setWrapToString_js_1 = require("./_setWrapToString.js");
var toInteger_js_1 = require("./toInteger.js");
var FUNC_ERROR_TEXT = 'Expected a function';
var WRAP_BIND_FLAG = 1, WRAP_BIND_KEY_FLAG = 2, WRAP_CURRY_FLAG = 8, WRAP_CURRY_RIGHT_FLAG = 16, WRAP_PARTIAL_FLAG = 32, WRAP_PARTIAL_RIGHT_FLAG = 64;
var nativeMax = Math.max;
function createWrap(func, bitmask, thisArg, partials, holders, argPos, ary, arity) {
    var isBindKey = bitmask & WRAP_BIND_KEY_FLAG;
    if (!isBindKey && typeof func != 'function') {
        throw new TypeError(FUNC_ERROR_TEXT);
    }
    var length = partials ? partials.length : 0;
    if (!length) {
        bitmask &= ~(WRAP_PARTIAL_FLAG | WRAP_PARTIAL_RIGHT_FLAG);
        partials = holders = undefined;
    }
    ary = ary === undefined ? ary : nativeMax(toInteger_js_1.default(ary), 0);
    arity = arity === undefined ? arity : toInteger_js_1.default(arity);
    length -= holders ? holders.length : 0;
    if (bitmask & WRAP_PARTIAL_RIGHT_FLAG) {
        var partialsRight = partials, holdersRight = holders;
        partials = holders = undefined;
    }
    var data = isBindKey ? undefined : _getData_js_1.default(func);
    var newData = [
        func, bitmask, thisArg, partials, holders, partialsRight, holdersRight,
        argPos, ary, arity
    ];
    if (data) {
        _mergeData_js_1.default(newData, data);
    }
    func = newData[0];
    bitmask = newData[1];
    thisArg = newData[2];
    partials = newData[3];
    holders = newData[4];
    arity = newData[9] = newData[9] === undefined
        ? (isBindKey ? 0 : func.length)
        : nativeMax(newData[9] - length, 0);
    if (!arity && bitmask & (WRAP_CURRY_FLAG | WRAP_CURRY_RIGHT_FLAG)) {
        bitmask &= ~(WRAP_CURRY_FLAG | WRAP_CURRY_RIGHT_FLAG);
    }
    if (!bitmask || bitmask == WRAP_BIND_FLAG) {
        var result = _createBind_js_1.default(func, bitmask, thisArg);
    }
    else if (bitmask == WRAP_CURRY_FLAG || bitmask == WRAP_CURRY_RIGHT_FLAG) {
        result = _createCurry_js_1.default(func, bitmask, arity);
    }
    else if ((bitmask == WRAP_PARTIAL_FLAG || bitmask == (WRAP_BIND_FLAG | WRAP_PARTIAL_FLAG)) && !holders.length) {
        result = _createPartial_js_1.default(func, bitmask, thisArg, partials);
    }
    else {
        result = _createHybrid_js_1.default.apply(undefined, newData);
    }
    var setter = data ? _baseSetData_js_1.default : _setData_js_1.default;
    return _setWrapToString_js_1.default(setter(result, newData), func, bitmask);
}
exports.default = createWrap;
