"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _MapCache_js_1 = require("./_MapCache.js");
var FUNC_ERROR_TEXT = 'Expected a function';
function memoize(func, resolver) {
    if (typeof func != 'function' || (resolver != null && typeof resolver != 'function')) {
        throw new TypeError(FUNC_ERROR_TEXT);
    }
    var memoized = function () {
        var args = arguments, key = resolver ? resolver.apply(this, args) : args[0], cache = memoized.cache;
        if (cache.has(key)) {
            return cache.get(key);
        }
        var result = func.apply(this, args);
        memoized.cache = cache.set(key, result) || cache;
        return result;
    };
    memoized.cache = new (memoize.Cache || _MapCache_js_1.default);
    return memoized;
}
memoize.Cache = _MapCache_js_1.default;
exports.default = memoize;
