"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var assignInWith_js_1 = require("./assignInWith.js");
var attempt_js_1 = require("./attempt.js");
var _baseValues_js_1 = require("./_baseValues.js");
var _customDefaultsAssignIn_js_1 = require("./_customDefaultsAssignIn.js");
var _escapeStringChar_js_1 = require("./_escapeStringChar.js");
var isError_js_1 = require("./isError.js");
var _isIterateeCall_js_1 = require("./_isIterateeCall.js");
var keys_js_1 = require("./keys.js");
var _reInterpolate_js_1 = require("./_reInterpolate.js");
var templateSettings_js_1 = require("./templateSettings.js");
var toString_js_1 = require("./toString.js");
var reEmptyStringLeading = /\b__p \+= '';/g, reEmptyStringMiddle = /\b(__p \+=) '' \+/g, reEmptyStringTrailing = /(__e\(.*?\)|\b__t\)) \+\n'';/g;
var reEsTemplate = /\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/g;
var reNoMatch = /($^)/;
var reUnescapedString = /['\n\r\u2028\u2029\\]/g;
function template(string, options, guard) {
    var settings = templateSettings_js_1.default.imports._.templateSettings || templateSettings_js_1.default;
    if (guard && _isIterateeCall_js_1.default(string, options, guard)) {
        options = undefined;
    }
    string = toString_js_1.default(string);
    options = assignInWith_js_1.default({}, options, settings, _customDefaultsAssignIn_js_1.default);
    var imports = assignInWith_js_1.default({}, options.imports, settings.imports, _customDefaultsAssignIn_js_1.default), importsKeys = keys_js_1.default(imports), importsValues = _baseValues_js_1.default(imports, importsKeys);
    var isEscaping, isEvaluating, index = 0, interpolate = options.interpolate || reNoMatch, source = "__p += '";
    var reDelimiters = RegExp((options.escape || reNoMatch).source + '|' +
        interpolate.source + '|' +
        (interpolate === _reInterpolate_js_1.default ? reEsTemplate : reNoMatch).source + '|' +
        (options.evaluate || reNoMatch).source + '|$', 'g');
    var sourceURL = 'sourceURL' in options ? '//# sourceURL=' + options.sourceURL + '\n' : '';
    string.replace(reDelimiters, function (match, escapeValue, interpolateValue, esTemplateValue, evaluateValue, offset) {
        interpolateValue || (interpolateValue = esTemplateValue);
        source += string.slice(index, offset).replace(reUnescapedString, _escapeStringChar_js_1.default);
        if (escapeValue) {
            isEscaping = true;
            source += "' +\n__e(" + escapeValue + ") +\n'";
        }
        if (evaluateValue) {
            isEvaluating = true;
            source += "';\n" + evaluateValue + ";\n__p += '";
        }
        if (interpolateValue) {
            source += "' +\n((__t = (" + interpolateValue + ")) == null ? '' : __t) +\n'";
        }
        index = offset + match.length;
        return match;
    });
    source += "';\n";
    var variable = options.variable;
    if (!variable) {
        source = 'with (obj) {\n' + source + '\n}\n';
    }
    source = (isEvaluating ? source.replace(reEmptyStringLeading, '') : source)
        .replace(reEmptyStringMiddle, '$1')
        .replace(reEmptyStringTrailing, '$1;');
    source = 'function(' + (variable || 'obj') + ') {\n' +
        (variable
            ? ''
            : 'obj || (obj = {});\n') +
        "var __t, __p = ''" +
        (isEscaping
            ? ', __e = _.escape'
            : '') +
        (isEvaluating
            ? ', __j = Array.prototype.join;\n' +
                "function print() { __p += __j.call(arguments, '') }\n"
            : ';\n') +
        source +
        'return __p\n}';
    var result = attempt_js_1.default(function () {
        return Function(importsKeys, sourceURL + 'return ' + source)
            .apply(undefined, importsValues);
    });
    result.source = source;
    if (isError_js_1.default(result)) {
        throw result;
    }
    return result;
}
exports.default = template;
