"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _baseToString_js_1 = require("./_baseToString.js");
var _castSlice_js_1 = require("./_castSlice.js");
var _hasUnicode_js_1 = require("./_hasUnicode.js");
var isObject_js_1 = require("./isObject.js");
var isRegExp_js_1 = require("./isRegExp.js");
var _stringSize_js_1 = require("./_stringSize.js");
var _stringToArray_js_1 = require("./_stringToArray.js");
var toInteger_js_1 = require("./toInteger.js");
var toString_js_1 = require("./toString.js");
var DEFAULT_TRUNC_LENGTH = 30, DEFAULT_TRUNC_OMISSION = '...';
var reFlags = /\w*$/;
function truncate(string, options) {
    var length = DEFAULT_TRUNC_LENGTH, omission = DEFAULT_TRUNC_OMISSION;
    if (isObject_js_1.default(options)) {
        var separator = 'separator' in options ? options.separator : separator;
        length = 'length' in options ? toInteger_js_1.default(options.length) : length;
        omission = 'omission' in options ? _baseToString_js_1.default(options.omission) : omission;
    }
    string = toString_js_1.default(string);
    var strLength = string.length;
    if (_hasUnicode_js_1.default(string)) {
        var strSymbols = _stringToArray_js_1.default(string);
        strLength = strSymbols.length;
    }
    if (length >= strLength) {
        return string;
    }
    var end = length - _stringSize_js_1.default(omission);
    if (end < 1) {
        return omission;
    }
    var result = strSymbols
        ? _castSlice_js_1.default(strSymbols, 0, end).join('')
        : string.slice(0, end);
    if (separator === undefined) {
        return result + omission;
    }
    if (strSymbols) {
        end += (result.length - end);
    }
    if (isRegExp_js_1.default(separator)) {
        if (string.slice(end).search(separator)) {
            var match, substring = result;
            if (!separator.global) {
                separator = RegExp(separator.source, toString_js_1.default(reFlags.exec(separator)) + 'g');
            }
            separator.lastIndex = 0;
            while ((match = separator.exec(substring))) {
                var newEnd = match.index;
            }
            result = result.slice(0, newEnd === undefined ? end : newEnd);
        }
    }
    else if (string.indexOf(_baseToString_js_1.default(separator), end) != end) {
        var index = result.lastIndexOf(separator);
        if (index > -1) {
            result = result.slice(0, index);
        }
    }
    return result + omission;
}
exports.default = truncate;
